

#include <stdio.h>

#include "ram.hh"


using namespace vm68k ;


RAM :: RAM ( uint32_type startAddr , uint32_type memorySize , bool debug ) 
{
    this -> memorySize = memorySize ;
    this -> startAddr = startAddr ;
    assert ( memorySize > 0 ) ;
    assert ( startAddr >= 0 ) ;

    this -> memory = new unsigned char [ memorySize ] ;
    this -> debug = debug ;
}

RAM :: ~RAM ()
{
    delete[] memory ;
}

int RAM :: get_8(uint32_type address, function_code fc) const
{
    if ( debug )
    {
        printf ( "8 bit read from 0x%04x  - %02x\n" , address ,
            memory [ address - startAddr ] ) ; 
    }
    return memory [ address - startAddr ] ;
}

uint16_type RAM :: get_16(uint32_type address, function_code fc) const
{
    if ( debug )
    {
        printf ( "16 bit read from 0x%04x  - %02x %02x\n" , address ,
            memory [ address - startAddr ] ,
            memory [ address + 1 - startAddr ] ) ;
    }
    return (( (uint16_type) memory [ address - startAddr ] << 8 ) +
            (uint16_type) memory [ address + 1 - startAddr ] ) ;
}

uint32_type RAM :: get_32(uint32_type address, function_code fc) const
{
    if ( debug )
    {
        printf ( "32 bit read from 0x%04x  - %02x %02x %02x %02x\n" , address ,
            memory [ address - startAddr ] ,
            memory [ address + 1 - startAddr ] ,
            memory [ address + 2 - startAddr ] ,
            memory [ address + 3 - startAddr ] ) ;
    }
    return (( (uint32_type) memory [ address - startAddr ] << 24 ) +
            ( (uint32_type) memory [ address + 1 - startAddr ] << 16 ) +
            ( (uint32_type) memory [ address + 2 - startAddr ] << 8 ) +
            (uint32_type) memory [ address + 3 - startAddr ] ) ;
}

void RAM :: put_8(uint32_type address, int value, function_code fc) 
{
    if ( debug )
    {
        printf ( "Write 0x%02x to 0x%04x\n" , value , address ) ;
    }
    memory [ address - startAddr ] = (unsigned char) value ;
}

void RAM :: put_16(uint32_type address, uint16_type value, function_code fc) 
{
    if ( debug )
    {
        printf ( "Write 0x%04x to 0x%04x\n" , value , address ) ;
    }
    memory [ address - startAddr ] = (unsigned char) ( value >> 8 ) ;
    memory [ address + 1 - startAddr ] = (unsigned char) value ;
}

void RAM :: put_32(uint32_type address, uint32_type value, function_code fc) 
{
    if ( debug )
    {
        printf ( "Write 0x%08x to 0x%04x\n" , value , address ) ;
    }
    memory [ address - startAddr ] = (unsigned char) ( value >> 24 ) ;
    memory [ address + 1 - startAddr ] = (unsigned char) ( value >> 16 ) ;
    memory [ address + 2 - startAddr ] = (unsigned char) ( value >> 8 ) ;
    memory [ address + 3 - startAddr ] = (unsigned char) value ;
}


