
library ieee ;
use ieee . std_logic_1164 . all ;
use ieee . std_logic_arith . all ;
use ieee . std_logic_unsigned . all ;


entity dp_ram is 
    generic ( address_width : integer := 5 ;
              data_width : integer := 4 ) ;
    port ( clock : in std_logic ; 
     write : in std_logic ; 
     address1 : in std_logic_vector (( address_width - 1 ) downto 0 ) ;
     address2 : in std_logic_vector (( address_width - 1 ) downto 0 ) ; 
     data_in : in std_logic_vector (( data_width - 1 ) downto 0 ) ; 
     data_out1 : out std_logic_vector (( data_width - 1 ) downto 0 ) ; 
     data_out2 : out std_logic_vector (( data_width - 1 ) downto 0 ) ) ; 
end dp_ram ; 

architecture syn of dp_ram is
    constant max_address : integer := ( 2 ** address_width ) - 1 ;

    type ram_type is array ( max_address downto 0 ) 
                    of std_logic_vector (( data_width - 1 ) downto 0 ) ; 

    signal RAM : ram_type ; 
    signal read_address1 : std_logic_vector (( address_width - 1 ) downto 0 ) ; 
    signal read_address2 : std_logic_vector (( address_width - 1 ) downto 0 ) ; 
begin 
    ram_process : process ( clock ) is
    begin 
        if ( clock'event )
        and ( clock = '1' ) 
        then
            if ( write = '1' ) 
            then 
                RAM ( conv_integer ( address1 ) ) <= data_in ; 
            end if; 
            read_address1 <= address1 ; 
            read_address2 <= address2 ; 
        end if ; 
    end process ; 

    data_out1 <= RAM ( conv_integer ( read_address1 ) ) ; 
    data_out2 <= RAM ( conv_integer ( read_address2 ) ) ; 
end syn ; 


