

-- Clock processes


-- The clock controller is part of the debugging system.
-- The clock can be run slowly: the switch settings control the speed.
    clock_controller : process ( switches , fast_clock , 
            button_clock_event , slow_clock ) is
    begin
        if ( fast_clock = '1' )
        and ( fast_clock'event )
        then
            button_clock_event_clear_1 <= '0' ;
            reset <= '0' ;
            run_single_instruction <= '0' ;

            case switches ( 6 downto 5 ) is
            when "00" =>
                -- Reset the processor.
                reset <= '1' ;
                clock <= not clock ;
            when "01" =>
                -- Advance to the next clock edge on each button press
                if ( button_clock_event = '1' )
                then
                    clock <= not clock ;
                    button_clock_event_clear_1 <= '1' ;
                end if ;
            when others =>
                -- Run continuously at full-ish speed. 
                clock <= slow_clock ( 2 ) ;
            end case ;
            slow_clock <= slow_clock + 1 ;
        end if ;
    end process ;


-- This process debounces the button.
    button_debouncer : process ( fast_clock , button_clock_event_clear_1 ,
            button_clock_event_clear_2 , button , last_button ) is
    begin
        if ( fast_clock = '0' )
        and ( fast_clock'event )
        then
            if ( button_clock_event_clear_1 = '1' )
            or ( button_clock_event_clear_2 = '1' )
            then
                button_clock_event <= '0' ;
            end if ;
                
            if (( button xor last_button ) = '1' )
            then
                -- Button just changed.
                if ( button_state_stable ( 
                    button_state_stable'length - 1 ) = '1' )
                then
                    -- button was stable. generate a clock event.
                    button_clock_event <= '1' ;
                end if ;
                button_state_stable <= ( others => '0' ) ;
            else
                if ( button_state_stable ( 
                    button_state_stable'length - 1 ) = '0' )
                then
                    button_state_stable <= button_state_stable + 1 ;
                end if ;
            end if ;
            last_button <= button ;
        end if ;
    end process ;

