#ifndef OPCODE_DATABASE_H
#define OPCODE_DATABASE_H

#include <set>

#include "definitions.h"
#include "ndfa_dag.h"
#include "opcode_family.h"
#include "opcode_map_reader.h"
#include "state.h"
#include "state_machine.h"

class Opcode_Database : public Opcode_Map_Reader
{
public:
    Opcode_Database () ;

    virtual Accept_State * New_Accept_State ( 
                const char * cmp_bit_pattern , 
                const char * opcode_name , 
                const char * dea , 
                const char * micro_sub_name , 
                const char * optimisation_info , 
                const char * comment ) ;

    const char * Require_Opcode ( unsigned opcode ) ;

    /* First, read in the opcode map(s) by 1 or more call to Read_Opcode_Map.
     *  (which is inherited from Opcode_Map_Reader)
     * Then call Require_Opcode for every opcode that is needed.
     * Then call Finalise_DFA 
     * Call Generate_VHDL to produce the VHDL output for the decoder */

    void Finalise_DFA ( State_Machine * sm ) ;
    void Generate_VHDL ( FILE * fd )
            { ndfa . Generate_VHDL ( fd ) ; } ;


    Label_Set List_Required_Micro_Subroutines () ;

private:
    void Link_Subroutines ( State_Machine * sm ) ;


    Opcode_Set      opcode_set ;
    bool            finalised ;
} ;

#endif

