#ifndef OPCODE_FAMILY_H
#define OPCODE_FAMILY_H

#include <string.h>
#include <stdio.h>

#include <set>
#include <list>

#include "definitions.h"
#include "state_machine.h"
#include "optimisation.h"
#include "ndfa_accept_state.h"



struct Opcode_Record
{
    const char * opcode_name ;
    const char * comment ;
} ;


typedef list<Opcode_Record *> Opcode_Record_List;
typedef Opcode_Record_List::iterator Opcode_Record_List_Iterator ;


class State_Machine ;
class State ;

class Opcode_Family : public Accept_State
{
public:
    Opcode_Family ( const char * micro_sub_name ) ;
    virtual ~Opcode_Family () ;

    virtual void Print_Info ( FILE * fd , const char * old_tab_str ) ;
    virtual void Accept_State_To_VHDL ( FILE * fd , const char * indent ) ;


    void Add_Opcode ( const char * opcode_name , const char * comment ) ;
    const char * Get_Micro_Sub_Name ()
            { return micro_sub_name ; } ;
    void Link_Subroutine ( State_Machine * sm ) ;

private:

    Opcode_Record_List      opcodes_in_family ;
    const char *            micro_sub_name ;
    State *                 link ;
} ;


struct Opcode_Family_Compare
{
    bool operator () ( Opcode_Family * a1 , Opcode_Family * a2 ) const
    {
        return strcmp ( a1 -> Get_Micro_Sub_Name () ,
                        a2 -> Get_Micro_Sub_Name () ) < 0 ;
    }
} ;

typedef set<Opcode_Family *, Opcode_Family_Compare> Opcode_Set ;
typedef Opcode_Set::iterator Opcode_Set_Iterator ;

#endif

