
#include <string.h>
#include <regex.h>
#include <assert.h>
#include <ctype.h>

#include "utils.h"

int g_verbose_setting ;



/* This procedure deletes the newline character and everything
 * after it from the supplied string. If no newline character
 * is present, the string is unchanged. */
void Remove_Trailing_Newlines ( char * str )
{
    char * newline_index = index ( str , '\n' ) ;

    if ( newline_index != NULL )
    {
        newline_index [ 0 ] = '\0' ;
    }
}

/* Extracts a particular regular expression match substring from
 * match_str. An assertion fails if the regex function didn't find any
 * substring to match the specified pattern.. i.e. if the match
 * parameter is not valid.
 * 
 * Note: a deep copy of the substring is made that will need to be freed
 * later. */
char * Get_Regex_Match ( const char * match_str , regmatch_t * match )
{
    int     length_of_string = ( match -> rm_eo - match -> rm_so ) ;
    char *  substring ;

    assert ( match -> rm_so >= 0 ) ;
    assert ( length_of_string >= 0 ) ;

    substring = new char [ sizeof ( char ) * ( length_of_string + 1 ) ] ;

    memcpy ( substring , & match_str [ match -> rm_so ] , 
                        length_of_string ) ;
    substring [ length_of_string ] = '\0' ;

    return substring ;
}

/* Compute the minimum number of bits required to represent 
 * the integer parameter. */
int Get_Number_Of_Bits_Needed_For ( int parameter )
{
    if ( parameter <= 2 )
    {
        return 1 ;
    }

    int width = 0 ;

    while ( parameter > 0 )
    {
        parameter = parameter >> 1 ;
        width ++ ;
    }

    return width ;
}

/** Print_Binary
 *
 * Prints a number in binary. The given binary number is sent to the
 * specified stream, surrounded by double quotes.
 */
void Print_Binary ( FILE * fd , int number , int width )
{
    const char * s = Binary_To_String ( number , width ) ;

    fprintf ( fd , "\"%s\"" , s ) ;

    delete [] s ;
}

/** Binary_To_String
 *
 * Creates a new string, of length 'width+1', and puts the
 * given binary number in the string.
 */
const char * Binary_To_String ( int number , int width )
{
    char * str      = new char [ width + 1 ] ;

    str [ width ] = '\0' ;
    for ( int i = width - 1 ; i >= 0 ; i -- )
    {
        str [ ( width - 1 ) - i ] = (( number >> i ) & 1 ) ? '1' : '0' ;
    }
    return str ;
}


/** Copy_String
 *
 * The same as 'strdup', but uses 'new', not 'malloc'.
 */
const char * Copy_String ( const char * original )
{
    char * str      = new char [ strlen ( original ) + 1 ] ;

    strcpy ( str , original ) ;
    return str ;
}


/* Returns true if the string contains any non-whitespace characters */
bool String_Contains_Non_Whitespace ( const char * str ) 
{
    int     i ;

    for ( i = strlen ( str ) - 1 ; i >= 0 ; i -- )
    {
        if ( ! isspace ( str [ i ] ) )
        {
            return true ;
        }
    }
    return false ;
}


/* Removes whitespace characters from the beginning and end of the string */
void Remove_Whitespace_From_Ends ( char * str ) 
{
    unsigned        i = strlen ( str ) - 1 ;

    /* First remove whitespace from the end of the string. */
    for ( ; i >= 0 ; i -- )
    {
        if ( isspace ( str [ i ] ) )
        {
            str [ i ] = '\0' ;
        } else {
            break ;
        }
    }
    /* Now, remove whitespace from the beginning */
    for ( i = 0 ; i < strlen ( str ) ; i ++ )
    {
        if ( ! isspace ( str [ i ] ) )
        {
            break ;
        }
    }
    /* str[i] is the first non-whitespace character.
     * Move str[i] to str[0], copying the null terminator as well. */
    if (( i > 0 )
    && ( i < strlen ( str )) )
    {
        memmove ( & str [ 0 ] , & str [ i ] , strlen ( & str [ i ] ) + 1 ) ;
    }
}

